C Demonstration of how to use the optimization subroutine
C FMINSI with test function SPHDIS.
C
C   FMINSI - Fortran subroutines for unconstrained function minimization
C   Copyright (C) 1992, 2001  Hugo Pfoertner
C
C   This library is free software; you can redistribute it and/or
C   modify it under the terms of the GNU Lesser General Public
C   License as published by the Free Software Foundation; either
C   version 2.1 of the License, or (at your option) any later version.
C
C   This library is distributed in the hope that it will be useful,
C   but WITHOUT ANY WARRANTY; without even the implied warranty of
C   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
C   Lesser General Public License for more details.
C
C   You should have received a copy of the GNU Lesser General Public
C   License along with this library; if not, write to the Free Software
C   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
C   USA
C
C   Contact info: mailto:hugo@pfoertner.org
C   or use the information provided at http://www.pfoertner.org/
C
C Author: Hugo Pfoertner, Oberhaching, Germany
C
C Version History:
C 03.06.01 Initial version
C
      PARAMETER ( PI=3.14159265358, PD=PI/180.0 )
C Vectors for variables and convergence limits
      REAL X(4), EPS(4)
C External objective function of 4 variables to be minimized
      REAL SPHDIS
      EXTERNAL SPHDIS
C Common block to pass parameters to the objective function
      COMMON /SPHPAR/ OFFSET, RHO
C Common block to with flag for diagnostic printout
C and function call counter
      COMMON /ACTPRN/ IPRN, IFCCNT
C
C Start of loop over test cases
C
10    CONTINUE
C
C Preset parameters for the  objective function
C (offset of center of 2nd sphere on z axis and radius)
      OFFSET = 0.0
      RHO = 1.0
C
C Ask user for change of parameters
      WRITE (*,*) ' Enter OFFSET(Default 0), RHO(Default 1):'
      READ (*,*) OFFSET, RHO
      WRITE (*,*) ' OFFSET=', OFFSET, ' RHO=', RHO
C
C Test for termination (both parameters negative)
C
      IF ( OFFSET .LT. 0.0 .AND. RHO .LT. 0.0 ) STOP ' Test stopped'
C
C Input of starting values
C Set defaults
      PHI1 = 0.0
      THE1 = 0.0
      PHI2 = 90.0
      THE2 = 90.0
      WRITE (*,*) ' Enter PHI1(0), THE1(0), PHI2(90), THE2(90):'
      READ (*,*) PHI1, THE1, PHI2, THE2
C
C Preset tolerance vector
C
      DO 20 I = 1, 4
      EPS(I) = 0.001 * PI
20    CONTINUE
C
C Input of tolerance vector
      WRITE (*,1000) (EPS(I),I=1,4)
1000  FORMAT ( ' Change EPS ', 4(1PE8.2E1),
     &         ' or enter / to accept default:')
      READ (*,*) (EPS(I),I=1,4)
C
C Ask for IER (negative input causes display of diagnostic info)
      IER = 0
      WRITE (*,*) ' Enter -1 to activate execution trace, else /'
      READ (*,*) IER
C
C Convert coordinates
      X(1) = PHI1 * PD
      X(2) = THE1 * PD
      X(3) = PHI2 * PD
      X(4) = THE2 * PD
C
C Compute function value a starting point
      IPRN = 0
      FMIN = SPHDIS (4,X)
      WRITE (*,*) ' Starting point and equivalent EPS:'
      WRITE (*,1001) PHI1, THE1, PHI2, THE2, (EPS(I)/PD,i=1,4)
1001  FORMAT (1X, 4F9.3, /, 1X, 4F9.4)
      WRITE (*,*) ' Distance at starting point:', FMIN
C
C IPRN is used to pass the request for diagnostic printout to the
C objective function
      IPRN = IER
C
C Reset count of function calls
      IFCCNT = 0
C
C Call optimization subroutine
C
      CALL FMINSI ( 4, X, SPHDIS, EPS, FMIN, IER )
C
C Print results
      WRITE (*,1002) IER, IFCCNT, FMIN, (X(I)/PD,I=1,4)
1002  FORMAT ( ' Results: IER=', I2,',', I5, ' Function evaluations,',
     &         ' Minimum distance =', F15.6,/,' Position of minimum:',
     &        /,' PHI1=',F9.4,', THE1=',F9.4,
     &        /,' PHI2=',F9.4,', THE2=',F9.4 )
C
C Skip to next test case
      WRITE (*,*) ' Enter OFFSET=-1, RHO=-1 to quit'
      GOTO 10
C End of program
      END
